/* Automatically generated by po2tbl.sed from mc.pot.  */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "libgettext.h"

const struct _msg_ent _msg_tbl[] = {
  {"", 1},
  {" Emacs key: ", 2},
  {" Execute Macro ", 3},
  {" Press macro hotkey: ", 4},
  {" Insert Literal ", 5},
  {" Press any key: ", 6},
  {" There is no other panel to compare contents to ", 7},
  {"GNU Midnight Commander", 8},
  {"The GNOME edition of the Midnight Commander file manager.", 9},
  {" The Midnight Commander ", 10},
  {" Do you really want to quit the Midnight Commander? ", 11},
  {"&Yes", 12},
  {"&No", 13},
  {"Sort By", 14},
  {"Ignore case sensitivity.", 15},
  {"Name", 16},
  {"File Type", 17},
  {"Size", 18},
  {"Time Last Accessed", 19},
  {"Time Last Modified", 20},
  {"Time Last Changed", 21},
  {"Reverse the order.", 22},
  {"Enter name.", 23},
  {"Enter label for command:", 24},
  {"Find all core files", 25},
  {"Find rejects after patching", 26},
  {"Run Command", 27},
  {"Preset Commands", 28},
  {"Add", 29},
  {"Remove", 30},
  {"Run this Command", 31},
  {"Command: ", 32},
  {"Set Filter", 33},
  {"Show all files", 34},
  {".", 35},
  {"\
Enter a filter here for files in the panel view.\n\
\n\
For example:\n\
*.gif will show just gif images", 36},
  {"Enter a Regular Expression to filter files in the panel view.", 37},
  {" Open with...", 38},
  {"Enter extra arguments:", 39},
  {"Desktop entry properties", 40},
  {"Access Time", 41},
  {"Creation Time", 42},
  {"Group", 43},
  {"Group ID", 44},
  {"Inode Number", 45},
  {"Mode", 46},
  {"Modification Time", 47},
  {"Number of Hard Links", 48},
  {"Owner", 49},
  {"Permission", 50},
  {"Size (short)", 51},
  {"Type", 52},
  {"User ID", 53},
  {"Possible Columns", 54},
  {"Displayed Columns", 55},
  {"Custom View", 56},
  {"Warning", 57},
  {"Could not open %s; will not have desktop icons", 58},
  {"While running the mount/umount command", 59},
  {"While running the eject command", 60},
  {"Error", 61},
  {"Home directory", 62},
  {"Could not symlink %s to %s; will not have initial home desktop icon.", 63},
  {"\
Unable to locate the file:\n\
background-properties-capplet\n\
in your path.\n\
\n\
We are unable to set the background.", 64},
  {"_Terminal", 65},
  {"Launch a new terminal in the current directory", 66},
  {"_Directory...", 67},
  {"Creates a new directory", 68},
  {"_Launcher...", 69},
  {"Creates a new launcher", 70},
  {"Arrange Icons", 71},
  {"Create New Window", 72},
  {"Recreate Desktop Shortcuts", 73},
  {"Rescan Desktop", 74},
  {"Configure Background Image", 75},
  {"To: ", 76},
  {"Copying from: ", 77},
  {"Deleting file: ", 78},
  {"Files Exist", 79},
  {"\
Some of the files you are trying to copy already exist in the destination \
folder.  Please select the action to be performed.", 80},
  {"Prompt me before overwriting any file.", 81},
  {"Don't overwrite any files.", 82},
  {"Overwrite:", 83},
  {"Older files.", 84},
  {"Files only if size differs.", 85},
  {"All files.", 86},
  {"File Exists", 87},
  {"The target file already exists: %s", 88},
  {"Replace it?", 89},
  {"Copy", 90},
  {"Move", 91},
  {"Destination", 92},
  {"Find Destination Folder", 93},
  {"Copy as a background process", 94},
  {"Advanced Options", 95},
  {"Preserve symlinks", 96},
  {"Follow links.", 97},
  {"\
Selecting this will copy the files that symlinks point to instead of just \
copying the link.", 98},
  {"Preserve file attributes.", 99},
  {"Preserves the permissions and the UID/GID if possible", 100},
  {"Recursively copy subdirectories.", 101},
  {"If set, this will copy the directories recursively", 102},
  {"\
%s\n\
\n\
Directory not empty. Delete it recursively?", 103},
  {" Delete: ", 104},
  {"Do the same for the rest", 105},
  {"Move Progress", 106},
  {"Copy Progress", 107},
  {"Delete Progress", 108},
  {"File ", 109},
  {"is ", 110},
  {"done.", 111},
  {"Password:", 112},
  {"Password", 113},
  {"Symbolic Link", 114},
  {"Existing filename (filename symlink will point to):", 115},
  {"Symbolic link filename:", 116},
  {"_Move here", 117},
  {"_Copy here", 118},
  {"_Link here", 119},
  {"Cancel drag", 120},
  {"\
Could not stat %s\n\
%s", 121},
  {"Default set of icons not found, please check your installation", 122},
  {"_Icon View", 123},
  {"Switch view to an icon display", 124},
  {"_Brief View", 125},
  {"Switch view to show just file name and type", 126},
  {"_Detailed View", 127},
  {"Switch view to show detailed file statistics", 128},
  {"_Custom View", 129},
  {"Switch view to show user-defined statistics", 130},
  {"Icons", 131},
  {"Brief", 132},
  {"Detailed", 133},
  {"Custom", 134},
  {"Enter command to run", 135},
  {"\
Notice that if you choose to terminate the file manager, you will\n\
also terminate the GNOME desktop handler.\n\
\n\
Are you sure you want to exit?", 136},
  {"\
The file manager and the desktop handler are now terminating\n\
\n\
If you want to start up again the desktop handler or the file manager\n\
you can launch it from the Panel, or you can run the UNIX command `gmc'\n\
\n\
Press OK to terminate the application, or cancel to continue using it.", 137},
  {"_Copy...", 138},
  {"Copy files", 139},
  {"_Delete...", 140},
  {"Delete files", 141},
  {"_Move...", 142},
  {"Rename or move files", 143},
  {"Show directory sizes", 144},
  {"Shows the disk space used by each directory", 145},
  {"Select _All", 146},
  {"Select all files in the current Panel", 147},
  {"_Select Files...", 148},
  {"Select a group of files", 149},
  {"_Invert Selection", 150},
  {"Reverses the list of tagged files", 151},
  {"_Rescan Directory", 152},
  {"Rescan the directory contents", 153},
  {"_Sort By...", 154},
  {"Confirmation settings", 155},
  {"_Filter View...", 156},
  {"Global option settings", 157},
  {"_Find File...", 158},
  {"Locate files on disk", 159},
  {"_Edit mime types...", 160},
  {"Edits the MIME type bindings", 161},
  {"_Run Command...", 162},
  {"Runs a command", 163},
  {"_Run Command in panel...", 164},
  {"Run a command and put the results in a panel", 165},
  {"_Background jobs...", 166},
  {"List of background operations", 167},
  {"Exit", 168},
  {"Terminates the file manager and the desktop", 169},
  {"Rescan System Devices", 170},
  {"_Layout", 171},
  {"_Commands", 172},
  {"_Desktop", 173},
  {"_Help", 174},
  {"File/New/Directory...", 175},
  {"Full Name: ", 176},
  {"File Name", 177},
  {"File Type: ", 178},
  {"File Type: Symbolic Link", 179},
  {"Target Name: INVALID LINK", 180},
  {"Target Name: ", 181},
  {"File Type: Directory", 182},
  {"File Type: Character Device", 183},
  {"File Type: Block Device", 184},
  {"File Type: Socket", 185},
  {"File Type: FIFO", 186},
  {"File Size: ", 187},
  {" bytes", 188},
  {" KBytes  (", 189},
  {" bytes)", 190},
  {" MBytes  (", 191},
  {"File Size: N/A", 192},
  {"File Created on: ", 193},
  {"Last Modified on: ", 194},
  {"Last Accessed on: ", 195},
  {"URL:", 196},
  {"Caption:", 197},
  {"Open", 198},
  {"Use default Open action", 199},
  {"Drop Action", 200},
  {"View", 201},
  {"Use default Drop action", 202},
  {"Use default View action", 203},
  {"Edit", 204},
  {"Use default Edit action", 205},
  {"Icon", 206},
  {"File Actions", 207},
  {"Open action", 208},
  {"Needs terminal to run", 209},
  {"File Permissions", 210},
  {"Current mode: ", 211},
  {"Read", 212},
  {"Write", 213},
  {"Exec", 214},
  {"Special", 215},
  {"User", 216},
  {"Other", 217},
  {"Set UID", 218},
  {"Set GID", 219},
  {"Sticky", 220},
  {"File ownership", 221},
  {"URL", 222},
  {"Statistics", 223},
  {"Options", 224},
  {"Permissions", 225},
  {" Properties", 226},
  {"You entered an invalid username", 227},
  {"You must rename your file to something", 228},
  {"You cannot rename a file to something containing a '/' character", 229},
  {"Select an application to open \"", 230},
  {"Select a file to run with", 231},
  {"Applications", 232},
  {"Program to run", 233},
  {"Mount device", 234},
  {"Unmount device", 235},
  {"Eject device", 236},
  {"Open with...", 237},
  {"View Unfiltered", 238},
  {"Copy...", 239},
  {"Delete", 240},
  {"Move...", 241},
  {"Hard Link...", 242},
  {"Symlink...", 243},
  {"Edit Symlink...", 244},
  {"Properties...", 245},
  {"Show backup files", 246},
  {"Show hidden files", 247},
  {"Mix files and directories", 248},
  {"Use shell patterns instead of regular expressions", 249},
  {"Confirm when deleting file", 250},
  {"Confirm when overwriting files", 251},
  {"Confirm when executing files", 252},
  {"Show progress while operations are being performed", 253},
  {"VFS Timeout:", 254},
  {"Seconds", 255},
  {"Anonymous FTP password:", 256},
  {"Always use FTP proxy", 257},
  {"Fast directory reload", 258},
  {"Compute totals before copying files", 259},
  {"FTP directory cache timeout :", 260},
  {"Allow customization of icons in icon view", 261},
  {"File display", 262},
  {"Confirmation", 263},
  {"VFS", 264},
  {"Caching", 265},
  {"Preferences", 266},
  {"Filename", 267},
  {"Full name: ", 268},
  {"Command", 269},
  {"Command:", 270},
  {"Use terminal", 271},
  {"File mode (permissions)", 272},
  {"<Unknown>", 273},
  {"General", 274},
  {"Title", 275},
  {"Select icon", 276},
  {"Rescan Directory", 277},
  {"Reloads the current directory", 278},
  {"New folder", 279},
  {"Creates a new folder here", 280},
  {"All files", 281},
  {"Archives and compressed files", 282},
  {"RPM/DEB files", 283},
  {"Text/Document files", 284},
  {"HTML and SGML files", 285},
  {"Postscript and PDF files", 286},
  {"Spreadsheet files", 287},
  {"Image files", 288},
  {"Video/animation files", 289},
  {"Audio files", 290},
  {"C program files", 291},
  {"C++ program files", 292},
  {"Objective-C program files", 293},
  {"Scheme program files", 294},
  {"Assembler program files", 295},
  {"Misc. program files", 296},
  {"Font files", 297},
  {"Search: %s", 298},
  {"%s bytes in %d file", 299},
  {"%s bytes in %d files", 300},
  {"<readlink failed>", 301},
  {"Filter", 302},
  {"Back", 303},
  {"Go to the previously visited directory", 304},
  {"Up", 305},
  {"Go up a level in the directory heirarchy", 306},
  {"Forward", 307},
  {"Go to the next directory", 308},
  {"Rescan", 309},
  {"Rescan the current directory", 310},
  {"Home", 311},
  {"Go to your home directory", 312},
  {"Location:", 313},
  {"&Ok", 314},
  {"Offset 0x%08x", 315},
  {"Col %d", 316},
  {"%s bytes", 317},
  {"_Goto line", 318},
  {"Jump to a specified line number", 319},
  {"_Monitor file", 320},
  {"Monitor file growing", 321},
  {"Regexp search", 322},
  {"Regular expression search", 323},
  {"_Wrap", 324},
  {"Wrap the text", 325},
  {"_Parsed view", 326},
  {"_Formatted", 327},
  {"_Hex", 328},
  {"_Search", 329},
  {"ok", 330},
  {"cancel", 331},
  {"help", 332},
  {"yes", 333},
  {"no", 334},
  {"exit", 335},
  {"abort", 336},
  {" Error ", 337},
  {" Failed trying to open file for reading: ", 338},
  {" Cannot get size/permissions info on file: ", 339},
  {" Not an ordinary file: ", 340},
  {" File is too large: ", 341},
  {"\
 \n\
 Increase edit.h:MAXBUF and recompile the editor. ", 342},
  {" Error allocating memory ", 343},
  {"Quick save ", 344},
  {"Safe save ", 345},
  {"Do backups -->", 346},
  {"&Cancel", 347},
  {"Extension:", 348},
  {" Edit Save Mode ", 349},
  {" Save As ", 350},
  {" Warning ", 351},
  {" A file already exists with this name. ", 352},
  {"Overwrite", 353},
  {"Cancel", 354},
  {" Save as ", 355},
  {" Error trying to save file. ", 356},
  {" Delete macro ", 357},
  {" Error trying to open temp file ", 358},
  {" Error trying to open macro file ", 359},
  {"key '%d 0': ", 360},
  {" Error trying to overwrite macro file ", 361},
  {" Macro ", 362},
  {" Press the macro's new hotkey: ", 363},
  {" Save macro ", 364},
  {" Delete Macro ", 365},
  {" Load macro ", 366},
  {" Confirm save file? : ", 367},
  {" Save file ", 368},
  {"Save", 369},
  {"\
 Current text was modified without a file save. \n\
 Continue discards these changes. ", 370},
  {"Continue", 371},
  {" Load ", 372},
  {" Error trying to open file for reading ", 373},
  {" Block is large, you may not be able to undo this action. ", 374},
  {" Continue ", 375},
  {" Cancel ", 376},
  {"al&L", 377},
  {"&Skip", 378},
  {"&Replace", 379},
  {" Replace with: ", 380},
  {" Confirm replace ", 381},
  {"scanf &Expression", 382},
  {"replace &All", 383},
  {"pr&Ompt on replace", 384},
  {"&Backwards", 385},
  {"&Regular expression", 386},
  {"&Whole words only", 387},
  {"case &Sensitive", 388},
  {" Enter replacement argument order eg. 3,2,1,4 ", 389},
  {" Enter replacement string:", 390},
  {" Enter search string:", 391},
  {" Replace ", 392},
  {" Search ", 393},
  {" Enter search text : ", 394},
  {" Enter replace text : ", 395},
  {"\
You can enter regexp substrings with %s (not \\1, \\2 like sed) then use \
\"Enter...order\"", 396},
  {" Enter argument (or substring) order : ", 397},
  {"\
Enter the order of replacement of your scanf format specifiers or regexp \
substrings", 398},
  {" Whole words only ", 399},
  {" Case sensitive ", 400},
  {" Regular expression ", 401},
  {"See the regex man page for how to compose a regular expression", 402},
  {" Backwards ", 403},
  {"Warning: Searching backward can be slow", 404},
  {" Prompt on replace ", 405},
  {"Ask before making each replacement", 406},
  {" Replace all ", 407},
  {" Scanf expression ", 408},
  {"Allows entering of a C format string, see the scanf man page", 409},
  {"Begin search, Enter", 410},
  {"Abort this dialog, Esc", 411},
  {"Replace", 412},
  {"Skip", 413},
  {"Replace all", 414},
  {"\
 Invalid regular expression, or scanf expression with to many conversions ", 415},
  {" Error in replacement format string. ", 416},
  {" %ld replacements made. ", 417},
  {" Search string not found. ", 418},
  {" Quit ", 419},
  {"\
 Current text was modified without a file save. \n\
 Save with exit? ", 420},
  {" File was modified, Save with exit? ", 421},
  {"Cancel quit", 422},
  {" &Cancel quit ", 423},
  {" &Yes ", 424},
  {" &No ", 425},
  {" Copy to clipboard ", 426},
  {" Unable to save to file. ", 427},
  {" Cut to clipboard ", 428},
  {" Goto line ", 429},
  {" Enter line: ", 430},
  {" Save Block ", 431},
  {" Insert File ", 432},
  {" Insert file ", 433},
  {" Error trying to insert file. ", 434},
  {" Sort block ", 435},
  {" You must first highlight a block of text. ", 436},
  {" Run Sort ", 437},
  {" Enter sort options (see manpage) separated by whitespace: ", 438},
  {" Sort ", 439},
  {" Error trying to execute sort command ", 440},
  {" Sort returned non-zero: ", 441},
  {" Process block ", 442},
  {" Error trying to stat file ", 443},
  {" Mail ", 444},
  {" Copies to", 445},
  {" Subject", 446},
  {" To", 447},
  {" mail -s <subject> -c <cc> <to>", 448},
  {"Intuitive", 449},
  {"Emacs", 450},
  {"None", 451},
  {"Dynamic paragraphing", 452},
  {"Type writer wrap", 453},
  {"Word wrap line length : ", 454},
  {"Tab spacing : ", 455},
  {"synta&X highlighting", 456},
  {"confir&M before saving", 457},
  {"fill tabs with &Spaces", 458},
  {"&Return does autoindent", 459},
  {"&Backspace through tabs", 460},
  {"&Fake half tabs", 461},
  {"Wrap mode", 462},
  {"Key emulation", 463},
  {" Editor options ", 464},
  {"Error initialising editor.\n", 465},
  {"Help", 466},
  {"Mark", 467},
  {"Replac", 468},
  {"Search", 469},
  {"PullDn", 470},
  {"Quit", 471},
  {" Word wrap ", 472},
  {" Enter line length, 0 for off: ", 473},
  {" About ", 474},
  {"\
\n\
                 Cooledit  v2.1\n\
\n\
 Copyright (C) 1996 the Free Software Foundation\n\
\n\
       A user friendly text editor written\n\
           for the Midnight Commander.\n", 475},
  {"&Open/load...     C-o", 476},
  {"&New              C-n", 477},
  {"&Save              F2", 478},
  {"save &As...       F12", 479},
  {"&Insert file...   F15", 480},
  {"copy to &File...  C-f", 481},
  {"a&Bout...            ", 482},
  {"&Quit             F10", 483},
  {"&New            C-x k", 484},
  {"copy to &File...     ", 485},
  {"&Toggle Mark       F3", 486},
  {"&Mark Columns    S-F3", 487},
  {"toggle &Ins/overw Ins", 488},
  {"&Copy              F5", 489},
  {"&Move              F6", 490},
  {"&Delete            F8", 491},
  {"&Undo             C-u", 492},
  {"&Beginning     C-PgUp", 493},
  {"&End           C-PgDn", 494},
  {"&Search...         F7", 495},
  {"search &Again     F17", 496},
  {"&Replace...        F4", 497},
  {"&Goto line...            M-l", 498},
  {"insert &Literal...       C-q", 499},
  {"&Refresh screen          C-l", 500},
  {"&Start record macro      C-r", 501},
  {"&Finish record macro...  C-r", 502},
  {"&Execute macro...   C-a, KEY", 503},
  {"delete macr&O...            ", 504},
  {"insert &Date/time           ", 505},
  {"format p&Aragraph        M-p", 506},
  {"'ispell' s&Pell check    C-p", 507},
  {"sor&T...                 M-t", 508},
  {"'indent' &C Formatter    F19", 509},
  {"&Mail...                    ", 510},
  {"&Execute macro... C-x e, KEY", 511},
  {"'ispell' s&Pell check    M-$", 512},
  {"&General...  ", 513},
  {"&Save mode...", 514},
  {"&Layout...", 515},
  {" File ", 516},
  {" Edit ", 517},
  {" Sear/Repl ", 518},
  {" Command ", 519},
  {" Options ", 520},
  {"Open...\tC-o", 521},
  {"New\tC-n", 522},
  {"Save\tF2", 523},
  {"Save as...\tF12", 524},
  {"Insert file...\tF15", 525},
  {"Copy to file...\tC-f", 526},
  {"Disk operations", 527},
  {"Toggle mark\tF3", 528},
  {"Toggle mark columns\tC-b", 529},
  {"Toggle book mark\tC-M-Ins", 530},
  {"Previous book mark\tC-M-Up", 531},
  {"Next book mark\tC-M-Down", 532},
  {"Flush book marks", 533},
  {"Toggle insert/overwrite\tIns", 534},
  {"Copy block to cursor\tF5", 535},
  {"Move block to cursor\tF6", 536},
  {"Delete block\tF8/C-Del", 537},
  {"Copy block to clipbrd\tC-Ins", 538},
  {"Cut block to clipbrd\tS-Del", 539},
  {"Paste block from clipbrd\tS-Ins", 540},
  {"Selection history\tM-Ins", 541},
  {"Undo\tC-BackSpace", 542},
  {"Manipulating blocks of text", 543},
  {" Srch/Replce ", 544},
  {"Search...\tF7", 545},
  {"Search again\tF17", 546},
  {"Replace...\tF4", 547},
  {"Replace again\tF14", 548},
  {"Search for and replace text", 549},
  {"Goto line...\tM-l", 550},
  {"Start record macro\tC-r", 551},
  {"Finish record macro...\tC-r", 552},
  {"Execute macro...\tC-a, KEY", 553},
  {"Delete macro...\t", 554},
  {"Insert date/time\tC-d", 555},
  {"Format paragraph\tM-p", 556},
  {"Refresh display\tC-l", 557},
  {"Macros and internal commands", 558},
  {"&Dismiss", 559},
  {" Enter file name: ", 560},
  {"The Quick Brown Fox Jumps Over The Lazy Dog", 561},
  {"\
gtkedit.c: HOME environment variable not set and no passwd entry - aborting\n", 562},
  {"Clear the edit buffer", 563},
  {"Insert File", 564},
  {"Insert text from a file", 565},
  {"Copy to file", 566},
  {"copy a block to a file", 567},
  {"Search/Replace", 568},
  {" Load syntax file ", 569},
  {" File access error ", 570},
  {" Error in file %s on line %d ", 571},
  {"&Set", 572},
  {"S&kip", 573},
  {"Set &all", 574},
  {"owner", 575},
  {"group", 576},
  {"other", 577},
  {"On", 578},
  {"Flag", 579},
  {"%6d of %d", 580},
  {" Chown advanced command ", 581},
  {"\
 Couldn't chmod \"%s\" \n\
 %s ", 582},
  {"\
 Couldn't chown \"%s\" \n\
 %s ", 583},
  {" Oops... ", 584},
  {" I can't run the Advanced Chown command on an extfs ", 585},
  {" I can't run the Advanced Chown command on a tarfs ", 586},
  {"Background process:", 587},
  {" Background process error ", 588},
  {" Child died unexpectedly ", 589},
  {" Unknown error in child ", 590},
  {" Background protocol error ", 591},
  {"\
 Background process sent us a request for more arguments \n\
 than we can handle. \n", 592},
  {" Listing mode ", 593},
  {"&Full file list", 594},
  {"&Brief file list", 595},
  {"&Long file list", 596},
  {"&User defined:", 597},
  {"&Icon view", 598},
  {"user &Mini status", 599},
  {"Listing mode", 600},
  {"&Reverse", 601},
  {"case sensi&tive", 602},
  {"Sort order", 603},
  {" confirm &Exit ", 604},
  {" confirm e&Xecute ", 605},
  {" confirm o&Verwrite ", 606},
  {" confirm &Delete ", 607},
  {" Confirmation ", 608},
  {"Full 8 bits output", 609},
  {"ISO 8859-1", 610},
  {"7 bits", 611},
  {"F&ull 8 bits input", 612},
  {" Display bits ", 613},
  {"&Always use ftp proxy", 614},
  {"sec", 615},
  {"ftpfs directory cache timeout:", 616},
  {"ftp anonymous password:", 617},
  {"Timeout for freeing VFSs:", 618},
  {" Virtual File System Setting ", 619},
  {"Quick cd", 620},
  {"cd", 621},
  {"Symbolic link", 622},
  {"Running ", 623},
  {"Stopped", 624},
  {"&Stop", 625},
  {"&Resume", 626},
  {"&Kill", 627},
  {"Background Jobs", 628},
  {"execute/search by others", 629},
  {"write by others", 630},
  {"read by others", 631},
  {"execute/search by group", 632},
  {"write by group", 633},
  {"read by group", 634},
  {"execute/search by owner", 635},
  {"write by owner", 636},
  {"read by owner", 637},
  {"sticky bit", 638},
  {"set group ID on execution", 639},
  {"set user ID on execution", 640},
  {"C&lear marked", 641},
  {"S&et marked", 642},
  {"&Marked all", 643},
  {"Permissions (Octal)", 644},
  {"Owner name", 645},
  {"Group name", 646},
  {" Chmod command ", 647},
  {" Permission ", 648},
  {"Use SPACE to change", 649},
  {"an option, ARROW KEYS", 650},
  {"to move between options", 651},
  {"and T or INS to mark", 652},
  {"Chmod command", 653},
  {" I can't run the Chmod command on an extfs ", 654},
  {"Set &users", 655},
  {"Set &groups", 656},
  {" Name ", 657},
  {" Owner name ", 658},
  {" Group name ", 659},
  {" Size ", 660},
  {" Chown command ", 661},
  {" User name ", 662},
  {"<Unknown user>", 663},
  {"<Unknown group>", 664},
  {" I can't run the Chown command on an extfs ", 665},
  {" I can't run the Chown command on a tarfs ", 666},
  {" Can not fetch a local copy of %s ", 667},
  {" CD ", 668},
  {"Files tagged, want to cd?", 669},
  {"Could not change directory", 670},
  {" View file ", 671},
  {" Filename:", 672},
  {" Filtered view ", 673},
  {" Filter command and arguments:", 674},
  {"Create a new Directory", 675},
  {" Enter directory name:", 676},
  {" Filter ", 677},
  {" Set expression for filtering filenames", 678},
  {" Select ", 679},
  {"  Malformed regular expression  ", 680},
  {" Unselect ", 681},
  {"Extension file edit", 682},
  {" Which extension file you want to edit? ", 683},
  {"&User", 684},
  {"&System Wide", 685},
  {"Menu file edit", 686},
  {" Which menu file will you edit? ", 687},
  {"&Local", 688},
  {"&Home", 689},
  {" Compare directories ", 690},
  {" Select compare method: ", 691},
  {"&Quick", 692},
  {"&Size only", 693},
  {"&Thorough", 694},
  {" Both panels should be on the listing view mode to use this command ", 695},
  {" The command history is empty ", 696},
  {" Command history ", 697},
  {"\
 Not an xterm or Linux console; \n\
 the panels cannot be toggled. ", 698},
  {"Type `exit' to return to the Midnight Commander", 699},
  {"\
 Couldn't stat %s \n\
 %s ", 700},
  {" Link ", 701},
  {" to:", 702},
  {" link: %s ", 703},
  {" symlink: %s ", 704},
  {" Symlink `%s' points to: ", 705},
  {" Edit symlink ", 706},
  {" edit symlink, unable to remove %s: %s ", 707},
  {" edit symlink: %s ", 708},
  {" Link symbolically ", 709},
  {" Relative symlink ", 710},
  {" relative symlink: %s ", 711},
  {" Enter machine name (F1 for details): ", 712},
  {" Could not chdir to %s ", 713},
  {" Link to a remote machine ", 714},
  {" FTP to machine ", 715},
  {" Socket source routing setup ", 716},
  {" Enter host name to use as a source routing hop: ", 717},
  {" Host name ", 718},
  {" Error while looking up IP address ", 719},
  {" Undelete files on an ext2 file system ", 720},
  {"\
 Enter device (without /dev/) to undelete\n\
   files on: (F1 for details)", 721},
  {" Setup saved to ~/", 722},
  {" Setup ", 723},
  {"\
 Can't chdir to '%s' \n\
 %s ", 724},
  {" You can not execute commands on non-local filesystems", 725},
  {"\
\n\
\n\
\n\
refresh stack underflow!\n\
\n\
\n", 726},
  {"&Unsorted", 727},
  {"&Name", 728},
  {"&Extension", 729},
  {"&Modify time", 730},
  {"&Access time", 731},
  {"&Change time", 732},
  {"&Size", 733},
  {"&Inode", 734},
  {"&Type", 735},
  {"&Links", 736},
  {"N&GID", 737},
  {"N&UID", 738},
  {"&Owner", 739},
  {"&Group", 740},
  {"File exists but can not be stat-ed: %s %s", 741},
  {"Unknown", 742},
  {"\
 Can't create temporary command file \n\
 %s ", 743},
  {" Parameter ", 744},
  {" Can't execute commands on a Virtual File System directory ", 745},
  {" file error", 746},
  {"Format of the ", 747},
  {" file error ", 748},
  {"Format of the ~/", 749},
  {"\
 file has changed\n\
with version 3.0. You may want either to\n\
copy it from ", 750},
  {"\
mc.ext or use that\n\
file as an example of how to write it.\n", 751},
  {"mc.ext will be used for this moment.", 752},
  {" Copy ", 753},
  {" Move ", 754},
  {" Delete ", 755},
  {" Invalid target mask ", 756},
  {" Could not make the hardlink ", 757},
  {"\
 Cannot read source link \"%s\" \n\
 %s ", 758},
  {"\
 Cannot make stable symlinks across non-local filesystems: \n\
\n\
 Option Stable Symlinks will be disabled ", 759},
  {"\
 Cannot create target symlink \"%s\" \n\
 %s ", 760},
  {"\
 Cannot overwrite directory \"%s\" \n\
 %s ", 761},
  {"\
 Cannot stat source file \"%s\" \n\
 %s ", 762},
  {" `%s' and `%s' are the same file. ", 763},
  {"\
 Cannot create special file \"%s\" \n\
 %s ", 764},
  {"\
 Cannot chown target file \"%s\" \n\
 %s ", 765},
  {"\
 Cannot chmod target file \"%s\" \n\
 %s ", 766},
  {"\
 Cannot open source file \"%s\" \n\
 %s ", 767},
  {" Reget failed, about to overwrite file ", 768},
  {"\
 Cannot fstat source file \"%s\" \n\
 %s ", 769},
  {"\
 Cannot create target file \"%s\" \n\
 %s ", 770},
  {"\
 Cannot fstat target file \"%s\" \n\
 %s ", 771},
  {"\
 Cannot read source file \"%s\" \n\
 %s ", 772},
  {"\
 Cannot write target file \"%s\" \n\
 %s ", 773},
  {"(stalled)", 774},
  {"\
 Cannot close source file \"%s\" \n\
 %s ", 775},
  {"\
 Cannot close target file \"%s\" \n\
 %s ", 776},
  {"Incomplete file was retrieved. Keep it?", 777},
  {"&Delete", 778},
  {"&Keep", 779},
  {"\
 Cannot stat source directory \"%s\" \n\
 %s ", 780},
  {"\
 Source directory \"%s\" is not a directory \n\
 %s ", 781},
  {"\
 Cannot copy cyclic symbolic link \n\
 `%s' ", 782},
  {"\
 Cannot create target directory \"%s\" \n\
 %s ", 783},
  {"\
 Cannot chown target directory \"%s\" \n\
 %s ", 784},
  {"\
 Cannot stat file \"%s\" \n\
 %s ", 785},
  {" `%s' and `%s' are the same file ", 786},
  {" Cannot overwrite directory `%s' ", 787},
  {"\
 Cannot move file \"%s\" to \"%s\" \n\
 %s ", 788},
  {"\
 Cannot remove file \"%s\" \n\
 %s ", 789},
  {" Cannot overwrite directory \"%s\" %s ", 790},
  {" Cannot overwrite file \"%s\" %s ", 791},
  {"\
 Cannot move directory \"%s\" to \"%s\" \n\
 %s ", 792},
  {"\
 Cannot delete file \"%s\" \n\
 %s ", 793},
  {"\
 Cannot remove directory \"%s\" \n\
 %s ", 794},
  {" Internal error: get_file \n", 795},
  {"1Copy", 796},
  {"1Move", 797},
  {"1Delete", 798},
  {"%o %f \"%s\"%m", 799},
  {"%o %d %f%m", 800},
  {"%o %f \"%s\"%e", 801},
  {"%o %d %f%e", 802},
  {"file", 803},
  {"files", 804},
  {"directory", 805},
  {"directories", 806},
  {"files/directories", 807},
  {" with source mask:", 808},
  {" Can't operate on \"..\"! ", 809},
  {"Yes", 810},
  {"No", 811},
  {" Sorry, I could not put the job in background ", 812},
  {" Internal failure ", 813},
  {" Unknown file operation ", 814},
  {"\
 Destination \"%s\" must be a directory \n\
 %s ", 815},
  {"&Retry", 816},
  {"&Abort", 817},
  {"\
\n\
   Directory not empty.   \n\
   Delete it recursively? ", 818},
  {"\
\n\
   Background process: Directory not empty \n\
   Delete it recursively? ", 819},
  {"a&ll", 820},
  {"non&E", 821},
  {" Type 'yes' if you REALLY want to delete ", 822},
  {"all the directories ", 823},
  {" Recursive Delete ", 824},
  {" Background process: Recursive Delete ", 825},
  {"File", 826},
  {"Count", 827},
  {"Bytes", 828},
  {"Source", 829},
  {"Target", 830},
  {"Deleting", 831},
  {"Target file \"%s\" already exists!", 832},
  {"if &Size differs", 833},
  {"&Update", 834},
  {"Overwrite all targets?", 835},
  {"&Reget", 836},
  {"ap&Pend", 837},
  {"Overwrite this target?", 838},
  {"Target date: %s, size %d", 839},
  {"Source date: %s, size %d", 840},
  {" File exists ", 841},
  {" Background process: File exists ", 842},
  {"preserve &Attributes", 843},
  {"follow &Links", 844},
  {"to:", 845},
  {"&Using shell patterns", 846},
  {"&Background", 847},
  {"&Stable Symlinks", 848},
  {"&Dive into subdir if exists", 849},
  {"\
Invalid source pattern `%s' \n\
 %s ", 850},
  {"&Suspend", 851},
  {"Con&tinue", 852},
  {"&Chdir", 853},
  {"&Again", 854},
  {"&Quit", 855},
  {"Pane&lize", 856},
  {"&View - F3", 857},
  {"&Edit - F4", 858},
  {"Start at:", 859},
  {"Filename:", 860},
  {"Content: ", 861},
  {"&Tree", 862},
  {"Find File", 863},
  {"Grepping in %s", 864},
  {" Find/read ", 865},
  {" Problem reading from child ", 866},
  {"Finished", 867},
  {"Searching %s", 868},
  {"Suspend", 869},
  {"Restart", 870},
  {"Searching", 871},
  {"Find file", 872},
  {"Change to this directory", 873},
  {"Search again", 874},
  {"View this file", 875},
  {"Edit this file", 876},
  {"Send the results to a Panel", 877},
  {"Usage: fixhlp <width> <tocname>\n", 878},
  {"fixhlp: Cannot open toc for writing", 879},
  {"\
[Contents]\n\
 Topics:\n\
\n", 880},
  {"\
 Help file format error\n\
", 881},
  {" Internal bug: Double start of link area ", 882},
  {" Can't find node [Contents] in help file ", 883},
  {" Help ", 884},
  {"\
 Can't open file %s \n\
 %s ", 885},
  {" Can't find node %s in help file ", 886},
  {"Index", 887},
  {"Prev", 888},
  {"&Move", 889},
  {"&Remove", 890},
  {"&Append", 891},
  {"&Insert", 892},
  {"New &Entry", 893},
  {"New &Group", 894},
  {"&Up", 895},
  {"&Add current", 896},
  {"Change &To", 897},
  {"Subgroup - press ENTER to see list", 898},
  {"Active VFS directories", 899},
  {"Directory hotlist", 900},
  {" Directory path ", 901},
  {" Directory label ", 902},
  {"Moving ", 903},
  {"New hotlist entry", 904},
  {"Directory label", 905},
  {"Directory path", 906},
  {" New hotlist group ", 907},
  {"Name of new group", 908},
  {"Label for \"%s\":", 909},
  {" Add to hotlist ", 910},
  {" Remove: ", 911},
  {"\
\n\
 Group not empty.\n\
 Remove it?", 912},
  {" Top level group ", 913},
  {"Hotlist is now kept in file ~/", 914},
  {"MC will load hotlist from ~/", 915},
  {"and then delete [Hotlist] section there", 916},
  {" Hotlist Load ", 917},
  {"MC was unable to write ~/", 918},
  {" file, your old hotlist entries were not deleted", 919},
  {"You have ~/", 920},
  {" file and [Hotlist] section in ~/", 921},
  {"Your ~/", 922},
  {" most probably was created\n", 923},
  {"\
by an earlier development version of MC\n\
and is more actual than ~/", 924},
  {"\
 entries\n\
\n", 925},
  {"\
You can choose between\n\
\n\
  Remove - remove old hotlist entries from ~/", 926},
  {"\
  Keep   - keep your old entries; you will be asked\n\
           the same question next time\n\
  Merge  - add old entries to hotlist as group \"Entries from ~/", 927},
  {"&Merge", 928},
  {" Entries from ~/", 929},
  {" file your old hotlist entries were not deleted", 930},
  {"Midnight Commander %s", 931},
  {"Free nodes %d (%d%%) of %d", 932},
  {"No node information", 933},
  {"Free space ", 934},
  {" (%d%%) of ", 935},
  {"No space information", 936},
  {"Type:      %s ", 937},
  {"non-local vfs", 938},
  {"Device:    %s", 939},
  {"Filesystem: %s", 940},
  {"Accessed:  %s", 941},
  {"Modified:  %s", 942},
  {"Created:   %s", 943},
  {"Size:      ", 944},
  {" (%d blocks)", 945},
  {"Owner:     %s/%s", 946},
  {"Links:     %d", 947},
  {"Mode:      %s (%04o)", 948},
  {"Location:  %Xh:%Xh", 949},
  {"File:       %s", 950},
  {"File:       None", 951},
  {"&Vertical", 952},
  {"&Horizontal", 953},
  {"&Xterm hintbar", 954},
  {"h&Intbar visible", 955},
  {"&Keybar visible", 956},
  {"command &Prompt", 957},
  {"show &Mini status", 958},
  {"menu&Bar visible", 959},
  {"&Equal split", 960},
  {"pe&Rmissions", 961},
  {"&File types", 962},
  {"&Save", 963},
  {" Layout ", 964},
  {" Panel split ", 965},
  {" Highlight... ", 966},
  {" Other options ", 967},
  {"output lines", 968},
  {"Layout", 969},
  {" Learn keys ", 970},
  {" Teach me a key ", 971},
  {"\
Please press the %s\n\
and then wait until this message disappears.\n\
\n\
Then, press it again to see if OK appears\n\
next to its button.\n\
\n\
If you want to escape, press a single Escape key\n\
and wait as well.", 972},
  {" Cannot accept this key ", 973},
  {" You have entered \"%s\"", 974},
  {"OK", 975},
  {"\
It seems that all your keys already\n\
work fine. That's great.", 976},
  {"&Discard", 977},
  {"\
Great! You have a complete terminal database!\n\
All your keys work well.", 978},
  {"Learn keys", 979},
  {"Press all the keys mentioned here. After you have done it, check", 980},
  {"which keys are not marked with OK.  Press space on the missing", 981},
  {"key, or click with the mouse to define it. Move around with Tab.", 982},
  {"\
 The Commander can't change to the directory that \n\
 the subshell claims you are in.  Perhaps you have \n\
 deleted your working directory, or given yourself \n\
 extra access permissions with the \"su\" command? ", 983},
  {"Press any key to continue...", 984},
  {" The shell is already running a command ", 985},
  {"&Listing mode...", 986},
  {"&Quick view     C-x q", 987},
  {"&Info           C-x i", 988},
  {"&Sort order...", 989},
  {"&Filter...", 990},
  {"&Network link...", 991},
  {"FT&P link...", 992},
  {"&Drive...       M-d", 993},
  {"&Rescan         C-r", 994},
  {"&User menu          F2", 995},
  {"&View               F3", 996},
  {"Vie&w file...         ", 997},
  {"&Filtered view     M-!", 998},
  {"&Edit               F4", 999},
  {"&Copy               F5", 1000},
  {"c&Hmod           C-x c", 1001},
  {"&Link            C-x l", 1002},
  {"&SymLink         C-x s", 1003},
  {"edit s&Ymlink  C-x C-s", 1004},
  {"ch&Own           C-x o", 1005},
  {"&Advanced chown       ", 1006},
  {"&Rename/Move        F6", 1007},
  {"&Mkdir              F7", 1008},
  {"&Delete             F8", 1009},
  {"&Quick cd          M-c", 1010},
  {"select &Group      M-+", 1011},
  {"u&Nselect group    M-\\", 1012},
  {"reverse selec&Tion M-*", 1013},
  {"e&Xit              F10", 1014},
  {"&Directory tree", 1015},
  {"&Find file            M-?", 1016},
  {"s&Wap panels          C-u", 1017},
  {"switch &Panels on/off C-o", 1018},
  {"&Compare directories  C-x d", 1019},
  {"e&Xternal panelize    C-x !", 1020},
  {"show directory s&Izes", 1021},
  {"command &History", 1022},
  {"di&Rectory hotlist    C-\\", 1023},
  {"&Active VFS list      C-x a", 1024},
  {"&Background jobs      C-x j", 1025},
  {"&Undelete files (ext2fs only)", 1026},
  {"&Listing format edit", 1027},
  {"&Extension file edit", 1028},
  {"&Menu file edit", 1029},
  {"&Configuration...", 1030},
  {"c&Onfirmation...", 1031},
  {"&Display bits...", 1032},
  {"learn &Keys...", 1033},
  {"&Virtual FS...", 1034},
  {"&Save setup", 1035},
  {" &Left ", 1036},
  {" &Above ", 1037},
  {" &File ", 1038},
  {" &Command ", 1039},
  {" &Options ", 1040},
  {" &Right ", 1041},
  {" &Below ", 1042},
  {" Information ", 1043},
  {"\
 Using the fast reload option may not reflect the exact \n\
 directory contents. In this cases you'll need to do a  \n\
 manual reload of the directory. See the man page for   \n\
 the details.                                           ", 1044},
  {"Menu", 1045},
  {"Thank you for using GNU Midnight Commander", 1046},
  {"with mouse support on xterm%s.\n", 1047},
  {" and the Linux console", 1048},
  {"The TERM environment variable is unset!\n", 1049},
  {"Library directory for the Midnight Commander: %s\n", 1050},
  {"\
Option -m is obsolete. Please look at Display Bits... in the Option's menu\n", 1051},
  {"Use to debug the background code", 1052},
  {"Request to run in color mode", 1053},
  {"Specifies a color configuration", 1054},
  {"Edits one file", 1055},
  {"Displays this help message", 1056},
  {"Displays a help screen on how to change the color scheme", 1057},
  {"Log ftp dialog to specified file", 1058},
  {"Obsolete", 1059},
  {"Requests to run in black and white", 1060},
  {"Disable mouse support in text version", 1061},
  {"Disables subshell support", 1062},
  {"Prints working directory at program exit", 1063},
  {"Resets soft keys on HP terminals", 1064},
  {"To run on slow terminals", 1065},
  {"Use stickchars to draw", 1066},
  {"Enables subshell support (default)", 1067},
  {"Tries to use termcap instead of terminfo", 1068},
  {"Displays the current version", 1069},
  {"Launches the file viewer on a file", 1070},
  {"Forces xterm features", 1071},
  {"Geometry for the window", 1072},
  {"GEOMETRY", 1073},
  {"No windows opened at startup", 1074},
  {"Display the directory that holds the .links startup files and exit", 1075},
  {"\
Couldn't open tty line. You have to run mc without the -P flag.\n\
On some systems you may want to run # `which mc`\n", 1076},
  {" Notice ", 1077},
  {"\
 The Midnight Commander configuration files \n\
 are now stored in the ~/.mc directory, the \n\
 files have been moved now\n", 1078},
  {"safe de&Lete", 1079},
  {"cd follows lin&Ks", 1080},
  {"advanced cho&Wn", 1081},
  {"l&Ynx-like motion", 1082},
  {"Animation", 1083},
  {"rotatin&G dash", 1084},
  {"co&Mplete: show all", 1085},
  {"&Use internal view", 1086},
  {"use internal ed&It", 1087},
  {"auto m&Enus", 1088},
  {"&Auto save setup", 1089},
  {"shell &Patterns", 1090},
  {"Compute &Totals", 1091},
  {"&Verbose operation", 1092},
  {"&Fast dir reload", 1093},
  {"mi&X all files", 1094},
  {"&Drop down menus", 1095},
  {"ma&Rk moves down", 1096},
  {"show &Hidden files", 1097},
  {"show &Backup files", 1098},
  {"&Never", 1099},
  {"on dumb &Terminals", 1100},
  {"alwa&Ys", 1101},
  {" Configure options ", 1102},
  {" Panel options ", 1103},
  {" Pause after run... ", 1104},
  {"Configure options", 1105},
  {"&Add new", 1106},
  {" External panelize ", 1107},
  {"External panelize", 1108},
  {"Other command", 1109},
  {" Add to external panelize ", 1110},
  {" Enter command label: ", 1111},
  {" I can't run external panelize while logged on a non local directory ", 1112},
  {"Find *.orig after patching", 1113},
  {"Find SUID and SGID programs", 1114},
  {"Cannot invoke command.", 1115},
  {"Pipe close failed", 1116},
  {"Show this help message", 1117},
  {"Display brief usage message", 1118},
  {"MTime", 1119},
  {"ATime", 1120},
  {"CTime", 1121},
  {"Perm", 1122},
  {"Nl", 1123},
  {"Inode", 1124},
  {"UID", 1125},
  {"GID", 1126},
  {"  %s bytes in %d file%s", 1127},
  {"Unknow tag on display format: ", 1128},
  {" Do you really want to execute? ", 1129},
  {" No action taken ", 1130},
  {"RenMov", 1131},
  {"Mkdir", 1132},
  {"\
 Can't chdir to %s \n\
 %s ", 1133},
  {"Couldn't open named pipe %s\n", 1134},
  {" The shell is still active. Quit anyway? ", 1135},
  {" There are stopped jobs.", 1136},
  {" Quit anyway? ", 1137},
  {"Warning: Couldn't change to %s.\n", 1138},
  {"\
Can't open the %s file for writing:\n\
%s\n", 1139},
  {"Copy \"%s\" directory to:", 1140},
  {"Move \"%s\" directory to:", 1141},
  {"\
 Can't stat the destination \n\
 %s ", 1142},
  {" The destination isn't a directory ", 1143},
  {"  Delete %s?  ", 1144},
  {"Static", 1145},
  {"Dynamc", 1146},
  {"\
 Can't chdir to \"%s\" \n\
 %s ", 1147},
  {"Forget", 1148},
  {"Rmdir", 1149},
  {"\
Can't write to the %s file:\n\
%s\n", 1150},
  {" Invalid shell pattern defination \"%c\". ", 1151},
  {" Debug ", 1152},
  {" ERROR: ", 1153},
  {" True:  ", 1154},
  {" False: ", 1155},
  {" Warning -- ignoring file ", 1156},
  {"\
File %s is not owned by root or you or is world writable.\n\
Using it may compromise your security", 1157},
  {" I can't run programs while logged on a non local directory ", 1158},
  {" User menu ", 1159},
  {"name_trunc: too big", 1160},
  {" Pipe failed ", 1161},
  {" Dup failed ", 1162},
  {"\
File: \n\
\n\
    ", 1163},
  {"\
\n\
\n\
has been modified, do you want to save the changes?\n", 1164},
  {" Save changes ", 1165},
  {" Can't spawn child program ", 1166},
  {" Empty output from child filter ", 1167},
  {" Could not open file ", 1168},
  {" Can't open file \"", 1169},
  {"\
 Can't stat file \n\
 ", 1170},
  {" Can't view: not a regular file ", 1171},
  {"File: %s", 1172},
  {"  [grow]", 1173},
  {"Searching for `%s'", 1174},
  {" Search string not found ", 1175},
  {" Invalid regular expression ", 1176},
  {"\
 The current line number is %d.\n\
 Enter the new line number:", 1177},
  {" Enter regexp:", 1178},
  {"Ascii", 1179},
  {"Hex", 1180},
  {"Line", 1181},
  {"RxSrch", 1182},
  {"EdText", 1183},
  {"EdHex", 1184},
  {"UnWrap", 1185},
  {"Wrap", 1186},
  {"HxSrch", 1187},
  {"Raw", 1188},
  {"Parse", 1189},
  {"Unform", 1190},
  {"Format", 1191},
  {"Function key 1", 1192},
  {"Function key 2", 1193},
  {"Function key 3", 1194},
  {"Function key 4", 1195},
  {"Function key 5", 1196},
  {"Function key 6", 1197},
  {"Function key 7", 1198},
  {"Function key 8", 1199},
  {"Function key 9", 1200},
  {"Function key 10", 1201},
  {"Function key 11", 1202},
  {"Function key 12", 1203},
  {"Function key 13", 1204},
  {"Function key 14", 1205},
  {"Function key 15", 1206},
  {"Function key 16", 1207},
  {"Function key 17", 1208},
  {"Function key 18", 1209},
  {"Function key 19", 1210},
  {"Function key 20", 1211},
  {"Backspace key", 1212},
  {"End key", 1213},
  {"Up arrow key", 1214},
  {"Down arrow key", 1215},
  {"Left arrow key", 1216},
  {"Right arrow key", 1217},
  {"Home key", 1218},
  {"Page Down key", 1219},
  {"Page Up key", 1220},
  {"Insert key", 1221},
  {"Delete key", 1222},
  {"Completion/M-tab", 1223},
  {"+ on keypad", 1224},
  {"- on keypad", 1225},
  {"* on keypad", 1226},
  {"Left arrow keypad", 1227},
  {"Right arrow keypad", 1228},
  {"Up arrow keypad", 1229},
  {"Down arrow keypad", 1230},
  {"Home on keypad", 1231},
  {"End on keypad", 1232},
  {"Page Down keypad", 1233},
  {"Page Up keypad", 1234},
  {"Insert on keypad", 1235},
  {"Delete on keypad", 1236},
  {"Enter on keypad", 1237},
  {"Slash on keypad", 1238},
  {"NumLock on keypad", 1239},
  {"\
Couldn't open %s archive\n\
%s", 1240},
  {"Inconsistent extfs archive", 1241},
  {"Sorry, we can not do password authenticated connections for now.", 1242},
  {"ftpfs: Disconnecting from %s", 1243},
  {" FTP: Password required for ", 1244},
  {" Proxy: Password required for ", 1245},
  {"ftpfs: sending proxy login name", 1246},
  {"ftpfs: sending proxy user password", 1247},
  {"ftpfs: proxy authentication succeeded", 1248},
  {"ftpfs: connected to %s", 1249},
  {"ftpfs: sending login name", 1250},
  {"ftpfs: sending user password", 1251},
  {"ftpfs: logged in", 1252},
  {"ftpfs: Login incorrect for user %s ", 1253},
  {" Could not set source routing (%s)", 1254},
  {"ftpfs: Invalid host name.", 1255},
  {"ftpfs: Invalid host address.", 1256},
  {"ftpfs: making connection to %s", 1257},
  {"ftpfs: connection interrupted by user", 1258},
  {"ftpfs: connection to server failed: %s", 1259},
  {"Waiting to retry... %d (Control-C to cancel)", 1260},
  {"ftpfs: could not setup passive mode for source routing", 1261},
  {"ftpfs: could not setup passive mode", 1262},
  {"ftpfs: aborting transfer.", 1263},
  {"ftpfs: abort error: %s", 1264},
  {"ftpfs: abort failed", 1265},
  {"ftpfs: CWD failed.", 1266},
  {"ftpfs: couldn't resolve symlink", 1267},
  {"Resolving symlink...", 1268},
  {"ftpfs: Reading FTP directory %s... (don't use UNIX ls options)", 1269},
  {"ftpfs: Reading FTP directory %s...", 1270},
  {"ftpfs: FAIL", 1271},
  {"ftpfs: reading FTP directory interrupt by user", 1272},
  {"ftpfs: got listing", 1273},
  {"ftpfs: failed", 1274},
  {"ftpfs: failed; nowhere to fallback to", 1275},
  {"ftpfs: storing file %d (%d)", 1276},
  {"\
~/.netrc file has not correct mode.\n\
Remove password or correct mode.", 1277},
  {" MCFS ", 1278},
  {" The server does not support this version ", 1279},
  {"\
 The remote server is not running on a system port \n\
 you need a password to log in, but the information may \n\
 not be safe on the remote side.  Continue? \n", 1280},
  {" Yes ", 1281},
  {" No ", 1282},
  {" The remote server is running on strange port. Giving up.\n", 1283},
  {" MCFS Password required ", 1284},
  {" Invalid password ", 1285},
  {" Too many open connections ", 1286},
  {"\
Couldn't open tar archive\n\
%s", 1287},
  {"Unexpected EOF on archive file", 1288},
  {"Inconsistent tar archive", 1289},
  {"\
Hmm,...\n\
%s\n\
doesn't look like a tar archive.", 1290},
};

int _msg_tbl_length = 1290;
